from pydantic import BaseModel


class ClipSavePayload(BaseModel):
    """The payload for all clip save endpoints.
    start and end are the timestamps of the final clip relative to the extended clip timstamps.
    E.g. An extended clip starts on second 10 and ends on second 100 of the source clip. The values for
    start and end in ClipSavePayload are 25 and 50. This will create a final clip that starts on second 35 (10 + 25)
    and ends on second 60 (10 + 50)."""

    start: float
    end: float

    def to_json(self):
        return {"start": self.start, "end": self.end}



from enum import Enum


class ClipDuration(Enum):
    """Duration options for the duration of a clip."""

    UNDER_30 = "UNDER_30"
    BETWEEN_30_AND_60 = "BETWEEN_30_AND_60"
    BETWEEN_60_AND_90 = "BETWEEN_60_AND_90"
    BETWEEN_90_AND_180 = "BETWEEN_90_AND_180"

    @staticmethod
    def is_within_range(clip_duration: "ClipDuration", duration: int) -> bool:
        """Check if the duration is within the range of this enum.
        Args:
            clip_duration (ClipDuration): The enum to check against.
            duration (int): The duration to check.
        Returns:
            bool: True if the duration is within the range, False otherwise.
        """

        match clip_duration:
            case ClipDuration.UNDER_30:
                return duration < 30
            case ClipDuration.BETWEEN_30_AND_60:
                return 30 <= duration <= 60
            case ClipDuration.BETWEEN_60_AND_90:
                return 60 <= duration <= 90
            case ClipDuration.BETWEEN_90_AND_180:
                return 90 <= duration <= 180

        raise ValueError(f"Invalid ClipDuration enum value: {clip_duration}")

    @staticmethod
    def is_over_range(clip_duration: "ClipDuration", duration: int) -> bool:
        """Check if the duration is over the range of this enum.
        Args:
            clip_duration (ClipDuration): The enum to check against.
            duration (int): The duration to check.
        Returns:
            bool: True if the duration is over the range, False otherwise.
        """

        match clip_duration:
            case ClipDuration.UNDER_30:
                return duration > 30
            case ClipDuration.BETWEEN_30_AND_60:
                return duration > 60
            case ClipDuration.BETWEEN_60_AND_90:
                return duration > 90
            case ClipDuration.BETWEEN_90_AND_180:
                return duration > 180



from abc import abstractmethod
import json

from pydantic import BaseModel


class AbstractTranscriptComponent(BaseModel):
    @abstractmethod
    def to_json(self) -> dict:
        """Converts the object to a dictionary."""

    @classmethod
    def from_dict(cls, d):
        """Converts a dictionary to an object."""

    def to_json_str(self) -> str:
        """Converts the object to a JSON string."""

        obj = self.to_json()
        return json.dumps(obj, ensure_ascii=False)

    @classmethod
    def from_json(cls, json_str):
        """Converts a JSON string to an object."""

        d = json.loads(json_str)
        return cls.from_dict(d)



from fastclip.enums.ClipFormat import ClipFormat
from fastclip.schemas.Word import Word
from fastclip.schemas.AbstractTranscriptComponent import AbstractTranscriptComponent
from fastclip.schemas.ClipPreviewPayload import ClipPreviewPayload



from fastclip.schemas.AbstractTranscriptComponent import (
    AbstractTranscriptComponent,
)


class Word(AbstractTranscriptComponent):
    """Represents a word in a transcript."""

    word: str
    punctuated_word: str
    start: float
    end: float
    is_keyword: bool = False

    def __eq__(self, other: "Word"):
        if not isinstance(other, Word):
            return False

        return (
            self.word == other.word
            and self.punctuated_word == other.punctuated_word
            and self.start == other.start
            and self.end == other.end
        )

    def __repr__(self):
        return f"Word(word='{self.word}', punctuated_word='{self.punctuated_word}', start={self.start}, end={self.end})"

    def to_json(self):
        return {
            "word": self.word,
            "punctuated_word": self.punctuated_word,
            "start": self.start,
            "end": self.end,
            "is_keyword": self.is_keyword,
        }

    @classmethod
    def from_dict(cls, d):
        return cls(
            word=d["word"],
            punctuated_word=d["punctuated_word"],
            start=d["start"],
            end=d["end"],
            is_keyword=d["is_keyword"] if "is_keyword" in d else False,
        )



from pydantic import BaseModel


class ClipInterface(BaseModel):
    """Interface that describes a clip"""

    id: int
    url: str
    duration: int
    # Represents the transcript of the non-extended clip
    transcript: str
    # TODO: Deprecate after the trimmer feature
    transcription: str

    # Represents the start and end time of the clip relative to the extended clip
    start: float
    end: float

    def to_json(self):
        return {
            "id": self.id,
            "transcript": self.transcript,
            "transcription": self.transcription,
            "url": self.url,
            "duration": self.duration,
            "start": self.start,
            "end": self.end,
        }



from distutils.core import setup

# Edit this to update the package version
VERSION = "0.1.0"

FULL_VERSION = VERSION.replace(".", "")

setup(
    name="fastclip",
    version=VERSION,
    license="MIT",
    description="A private library for fastclip",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Fastclip",
    author_email="hello+pip@fastclip.io",
    url="https://github.com/fastclip/library",
    download_url=f"https://github.com/fastclip/library/archive/v_{FULL_VERSION}.tar.gz",
    keywords=[
        "fastclip",
    ],
    install_requires=[
        "typing_extensions",
        "pydantic",
    ],
    packages=["fastclip", "fastclip.enums", "fastclip.schemas"],
)



from enum import Enum
from typing import List
from pydantic import BaseModel

from fastclip.enums.ClipFormat import ClipFormat
from fastclip.schemas.SubtitleSettings import SubtitleSettings
from fastclip.schemas.Word import Word

# The default duration in seconds to extend the clip by if the extend type is set to START or END
EXTEND_DURATION = 30


class ExtendType(Enum):
    """What type of clip extension to use.
    NONE: Clip preview timestamps are not changed
    START: Clip preview start timestamp is extended
    END: Clip preview end timestamp is extended"""

    NONE = "none"
    START = "start"
    END = "end"


class ClipPreviewPayload(BaseModel):
    """The payload for all clip preview endpoints.
    There are two concepts here:
    1. The clip: The actual clip that will be previewed
    2. The extended clip: The clip that will be previewed with the extension applied
    The extended clip allows the user to move the start or end of the clip to make it better.
    """

    # The start and end of the actual clip
    start: float
    end: float

    subtitles_settings: SubtitleSettings
    subtitles: List[Word] = []
    format: ClipFormat = ClipFormat.HORIZONTAL
    has_subtitles: bool = False

    # Extended clip properties
    extend: ExtendType = ExtendType.NONE
    extended_start: float = 0
    extended_end: float = 0

    def to_json(self):
        return {
            "start": self.start,
            "end": self.end,
            "subtitles_settings": self.subtitles_settings.to_json(),
            "subtitles": [word.to_json() for word in self.subtitles],
            "format": self.format.value,
            "has_subtitles": self.has_subtitles,
            "extended_start": self.extended_start,
            "extended_end": self.extended_end,
        }



from pydantic import BaseModel


class SubtitleSettings(BaseModel):
    font: str = "DejaVu Serif"
    font_size: int = 20

    def to_json(self):
        return {
            "font": self.font,
            "font_size": self.font_size,
        }
